# -*- coding: utf-8 -*-
from __future__ import absolute_import

__title__ = 'scrim'
__author__ = 'Dan Bradham'
__email__ = 'danielbradham@gmail.com'
__license__ = 'MIT'
__url__ = 'https://github.com/danbradham/scrim.git'
__description__ = 'Opaque Python CLI Wrapper'
__version__ = '0.1.0'

from scrim.api import *
from scrim.globals import *
from scrim.utils import *
